/* Copyright 1996-1999, Jan D. Wolter and Steven R. Weiss, All Rights Reserved.
 */

#ifndef _BT_COMMON_H
#define _BT_COMMON_H

#include <stdio.h>
#include "config.h"
#include "version.h"
#include <sys/types.h>
#include <utmp.h>

#ifdef STDC_HEADERS
#include <string.h>
#include <stdlib.h>
#else
#ifndef HAVE_STRCHR
#define strchr index
#define strrchr rindex
#endif
char *strchr(), *strdup(), *strrchr(), *strstr(), *getenv();
void *malloc(), *realloc();
#endif

#define DFLT_CFLIST BBS_DIR"dflt.cflist"
#define CONFLIST BBS_DIR"conflist"
#define CONFINDEX BBS_DIR"confindex"
#define CENSOR_LOG BBS_DIR"censored"
#define IDENT_FILE "id"

#define INDEX_SUBDIR "indexdir"
#define INDEX_PREFIX "@"

#define MAXSYMLEN  32	/* max length of symbol names in internal dictionary */
#define BFSZ 1024	/* standard buffer size for various uses */

/* If using real unix logins, use real login name length */
#ifdef UNIX_ACCOUNTS
#define ALLOW_CFDIR
#ifdef UT_NAMELEN
#ifdef MAX_LOGIN_LEN
#undef MAX_LOGIN_LEN
#endif
#define MAX_LOGIN_LEN UT_NAMELEN
#endif
#endif

#ifdef PICOSPAN_COMPAT
#define HIDE_HTML
#else
#define PARENT_POINTERS
#endif

#ifdef YAPP_COMPAT
#define HIDE_HTML
#define KEEP_ULIST
#define CONF_LOG
#endif

#ifdef YAPP3_COMPAT
#define HIDE_HTML
#define KEEP_ULIST
#define CONF_LOG
#define PART_DIR BBS_DIR"part/"
#define PART_DIR_LEVEL 2	/* 0, 1 or 2 only */
#endif

#if defined(SQL_PGSQL) || defined(SQL_MYSQL) || defined(SQL_MSQL) || defined(SQL_ORACLE) || defined(SQL_SYBASE)
#define USE_SQL
#endif

#ifdef EDITUSER
# define NOPWEDIT 0
#else
# define NOPWEDIT 1
#endif

#ifdef SHARE_AUTH_IDENT

#ifdef PASSWD_FILE
#define AUTH_FILE PASSWD_FILE
#endif

#ifdef DBM_NAM_FILE
#define AUTH_DBM DBM_NAM_FILE
#endif

#endif

#define DFLT_FW "cfadm"

#define TRUE    1
#define FALSE   0

/* Standardize some defines that may be missing on systems */

#ifdef _PATH_LASTLOG
#define LASTLOG _PATH_LASTLOG
#else
#define LASTLOG "/usr/adm/lastlog"
#endif

#ifndef UT_NAMESIZE
#define UT_NAMESIZE 8
#endif


#ifdef DEBUG_MEMORY
#define malloc(s) xmalloc(s)
#define realloc(p,s) xrealloc(p,s)
#define free(p) xfree(p)
#define MYSTRDUP
void *xmalloc(size_t);
void *xrealloc(void *,size_t);
void *xfree(void *);
#endif

/* Variadic functions.  Backtalk pretty largely assumes an ANSI C compiler,
 * so we could reasonably get away with just supporting stdarg.h.  The
 * varargs.h support, however, isn't especially in our way, so we've left
 * it in.  The support for systems that have neither has been cut out, since
 * it really did limit us, and probably wouldn't have worked well anyway.
 */

#if __STDC__
#include <stdarg.h>
#define VA_START(args, lastarg) va_start(args, lastarg)
#else
#include <varargs.h>
#define VA_START(args, lastarg) va_start(args)
#endif /* __STDC__ */


/* Snprintf() and friends.  Use Mark Martinec's versions if the operating
 * system doesn't supply suitable ones.  We never use the asnprintf() or
 * vasnprintf() functions defined there, because they depend on va_copy()
 * which we can't find on all systems.
 */

#if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF) || !defined(HAVE_VASPRINTF)

/* The following handles the case of having snprintf() but not vsnprintf(), by
 * causing us to use the Martinec versions of both. */
#ifndef HAVE_VSNPRINTF
#define PREFER_PORTABLE_SNPRINTF
#endif

#ifndef HAVE_VPRINTF
#ifndef HAVE_VASPRINTF
#define NEED_VASPRINTF	/* Might not work, if va_copy is unavailable */
#endif
#endif

#include "snprintf.h"
#endif

#endif /* _BT_COMMON_H */
