/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

/* SYSTEM DICTIONARY - This is the table that stores all backtalk built-in
 * functions and variables.
 *
 * If NO_FUNCS is defined at compile time, we compile a version of the system
 * dictionary that doesn't include the pointers to the functions.  This is used
 * by the compiler and the mksysdict program (which generates the sysdict.h
 * file of offsets into this program).
 */

#include "backtalk.h"

#include <stdio.h>

#include "array.h"
#include "userfunc.h"
#include "spell.h"
#include "session.h"

#define D(n,f,t,i) {n, f, {t, 0, (void *)i}},

#ifdef NO_FUNCS

#define F(n,f,t,i) {n, f, {t, 0, (void *)NULL}},
#define E(n,f,l,i) {n, f, {TK_FUNCTION, 0, (void *)NULL}},

#else

#define F(n,f,t,i) {n, f, {t, 0, (void *)i}},

#ifdef DYNALOAD
#include "dynaload.h"
#ifdef NEED_USCORE
#define E(n,f,l,i) {n, f, {TK_DYNAMIC, l, (void *)"_"#i}},
#else
#define E(n,f,l,i) {n, f, {TK_DYNAMIC, l, (void *)#i}},
#endif
#else
#define E(n,f,l,i) {n, f, {TK_FUNCTION, 0, (void *)i}},
#endif

#include "builtin.h"
#include "conf.h"
#include "dict.h"
#include "exec.h"
#include "file.h"
#include "format.h"
#include "math.h"
#include "stack.h"
#include "sel.h"
#include "search.h"
#include "regular.h"
#include "browser.h"
#include "http.h"
#include "ps_ulist.h"
#include "mod_conf.h"
#include "adm_user.h"
#endif /* NO_FUNCS */

Dict sysdict[]= {

    /* WARNING:  Must be in strict alphabetical order */

    F("!",		DT_CONSTANT, TK_FUNCTION, func_not)
    F("'",		DT_CONSTANT, TK_FUNCTION, func_concatinate)
    F("*",		DT_CONSTANT, TK_FUNCTION, func_mul)
    F("+",		DT_CONSTANT, TK_FUNCTION, func_add)
    F("-",		DT_CONSTANT, TK_FUNCTION, func_sub)
    F("/",		DT_CONSTANT, TK_FUNCTION, func_div)
    D("[",		DT_CONSTANT, TK_MARK,     0)
    F("]",		DT_CONSTANT, TK_FUNCTION, func_mkarray)
    D("`",		DT_CONSTANT, TK_MARK,	  1)
    F("abs",		DT_CONSTANT, TK_FUNCTION, func_abs)
    D("alias",		0,	     TK_STRING,   "")
    D("allowanon",	DT_CONSTANT, TK_INTEGER,  1)
    D("allowgraphics",	0,           TK_INTEGER,  1)
    F("aload",		DT_CONSTANT, TK_FUNCTION, func_aload)
    D("amadm",		DT_PROTECT,  TK_INTEGER,  0)
    D("amfw",		DT_PROTECT,  TK_INTEGER,  0)
    F("and",		DT_CONSTANT, TK_FUNCTION, func_and)
    D("anonymity",	DT_CONSTANT, TK_INTEGER,  0)
    F("asort",		DT_CONSTANT, TK_FUNCTION, func_asort)
    D("author_edit",	DT_CONSTANT, TK_INTEGER,  0)
    D("author_erase",	DT_CONSTANT, TK_INTEGER,  1)
    D("author_freeze",	DT_CONSTANT, TK_INTEGER,  1)
    D("author_hide",	DT_CONSTANT, TK_INTEGER,  1)
    D("author_kill",	DT_CONSTANT, TK_INTEGER,  1)
    D("author_retire",	DT_CONSTANT, TK_INTEGER,  1)
    D("author_retitle",	DT_CONSTANT, TK_INTEGER,  1)
    D("authorid",	0,	     TK_STRING,   "")
    D("authorname",	0,	     TK_STRING,   "")
    D("authoruid",	0,	     TK_INTEGER,  0)
    D("auto_recompile",	DT_PROTECT,  TK_INTEGER,  1)
    F("band",		DT_CONSTANT, TK_FUNCTION, func_band)
    D("bbsdir",		DT_CONSTANT, TK_STRING,   BBS_DIR)
    F("begin",		DT_CONSTANT, TK_FUNCTION, func_begin)
    D("blindfold",	0,	     TK_INTEGER,  0)
    F("bor",		DT_CONSTANT, TK_FUNCTION, func_bor)
    F("break",		DT_CONSTANT, TK_FUNCTION, func_break)
    F("browser",	DT_CONSTANT, TK_FUNCTION, func_browser)
    F("cache_conflist",	DT_CONSTANT, TK_FUNCTION, func_cache_conflist)
    F("call",		DT_CONSTANT, TK_FUNCTION, func_call)
    D("canspell",	DT_CONSTANT, TK_INTEGER,  SPELLCHECK)
    F("cap",		DT_CONSTANT, TK_FUNCTION, func_cap)
    F("caps",		DT_CONSTANT, TK_FUNCTION, func_caps)
    D("cgiquery",	DT_PROTECT,  TK_STRING,   "")
    F("cgiquote",	DT_CONSTANT, TK_FUNCTION, func_cgiquote)
    F("chain",		DT_CONSTANT, TK_FUNCTION, func_chain)
    F("changegroup",	DT_CONSTANT, TK_FUNCTION, func_changegroup)
    F("changename",	DT_CONSTANT, TK_FUNCTION, func_changename)
    F("changepass",	DT_CONSTANT, TK_FUNCTION, func_changepass)
    F("check_partic",	DT_CONSTANT, TK_FUNCTION, func_check_partic)
    F("chomp",		DT_CONSTANT, TK_FUNCTION, func_chomp)
    F("cleanhtml",	DT_CONSTANT, TK_FUNCTION, func_cleanhtml)
    F("clear",		DT_CONSTANT, TK_FUNCTION, func_clear)
    F("cleartomark",	DT_CONSTANT, TK_FUNCTION, func_cleartomark)
    F("clip",		DT_CONSTANT, TK_FUNCTION, func_clip)
    F("close_conf",	DT_CONSTANT, TK_FUNCTION, func_close_conf)
    D("conf",		0,	     TK_STRING,   "")
    F("conf_alias",	DT_CONSTANT, TK_FUNCTION, func_conf_alias)
    F("conf_bull",	DT_CONSTANT, TK_FUNCTION, func_conf_bull)
    F("conf_dir",	DT_CONSTANT, TK_FUNCTION, func_conf_dir)
    F("conf_index",	DT_CONSTANT, TK_FUNCTION, func_conf_index)
    F("conf_login",	DT_CONSTANT, TK_FUNCTION, func_conf_login)
    F("conf_logout",	DT_CONSTANT, TK_FUNCTION, func_conf_logout)
    F("conf_new",	DT_CONSTANT, TK_FUNCTION, func_conf_new)
    D("confdir",	0,           TK_STRING,	  "")
    D("conflist",	DT_CONSTANT, TK_STRING,   CONFLIST)
    D("conftitle",	0,	     TK_STRING,   "")
    F("constant",	DT_CONSTANT, TK_FUNCTION, func_constant)
    F("continue",	DT_CONSTANT, TK_FUNCTION, func_continue)
    F("copy",		DT_CONSTANT, TK_FUNCTION, func_copy)
    D("copyright",	DT_CONSTANT, TK_STRING,   COPYRIGHT_STRING)
    F("count",		DT_CONSTANT, TK_FUNCTION, func_count)
    F("count_sel",	DT_CONSTANT, TK_FUNCTION, func_count_sel)
    F("counttomark",	DT_CONSTANT, TK_FUNCTION, func_counttomark)
    D("cpass",		0,	     TK_STRING,	  "")
    D("csel",		DT_MULTI,    TK_STRING,	  "")
    F("ctime",		DT_CONSTANT, TK_FUNCTION, func_ctime)
    F("cvi",		DT_CONSTANT, TK_FUNCTION, func_cvi)
    F("cvn",		DT_CONSTANT, TK_FUNCTION, func_cvn)
    F("cvs",		DT_CONSTANT, TK_FUNCTION, func_cvs)
    F("cvscol",		DT_CONSTANT, TK_FUNCTION, func_cvscol)
    F("cvt",		DT_CONSTANT, TK_FUNCTION, func_cvt)
    D("date",		0,	     TK_TIME,     0)
    D("dbtype",		DT_CONSTANT, TK_STRING,   "")
    F("dec",		DT_CONSTANT, TK_FUNCTION, func_dec)
    F("def",		DT_CONSTANT, TK_FUNCTION, func_def)
    F("default",	DT_CONSTANT, TK_FUNCTION, func_default)
    F("defconstant",	DT_CONSTANT, TK_FUNCTION, func_defconstant)
    F("defined",	DT_CONSTANT, TK_FUNCTION, func_defined)
    E("del_conf",	DT_CONSTANT, LIB_CNFADM,  func_del_conf)
    F("delgroup",	DT_CONSTANT, TK_FUNCTION, func_delgroup)
    F("delgroupmem",	DT_CONSTANT, TK_FUNCTION, func_delgroupmem)
    F("dfltgroup",	DT_CONSTANT, TK_FUNCTION, func_dfltgroup)
    F("directory",	DT_CONSTANT, TK_FUNCTION, func_directory)
    F("dtime",		DT_CONSTANT, TK_FUNCTION, func_dtime)
    F("dumpstack",	DT_CONSTANT, TK_FUNCTION, func_dumpstack)
    F("dup",		DT_CONSTANT, TK_FUNCTION, func_dup)
    E("edit_resp",	DT_CONSTANT, LIB_CNFADM,  func_edit_resp)
    D("editdate",	0,	     TK_TIME,     0)
    D("editfrozen",	DT_CONSTANT, TK_INTEGER,  0)
    F("end",		DT_CONSTANT, TK_FUNCTION, func_end)
    F("eq",		DT_CONSTANT, TK_FUNCTION, func_eq)
    E("erase_resp",	DT_CONSTANT, LIB_CNFADM,  func_erase_resp)
    D("erased",		0,	     TK_INTEGER,  0)
    F("exch",		DT_CONSTANT, TK_FUNCTION, func_exch)
    F("exec",		DT_CONSTANT, TK_FUNCTION, func_exec)
    D("exec_limit",	0,	     TK_INTEGER,  0)
    F("exists",		DT_CONSTANT, TK_FUNCTION, func_exists)
    F("expand",		DT_CONSTANT, TK_FUNCTION, func_expand)
    F("expect_resp",	0,           TK_INTEGER,  0)
    D("expire_session",	DT_CONSTANT, TK_INTEGER,  3600)
    D("favicon",	0,	     TK_STRING,	  "")
    F("filedate",	DT_CONSTANT, TK_FUNCTION, func_filedate)
    F("first_partic",	DT_CONSTANT, TK_FUNCTION, func_first_partic)
    F("firstdir",	DT_CONSTANT, TK_FUNCTION, func_firstdir)
    F("firstuser",	DT_CONSTANT, TK_FUNCTION, func_firstuser)
    D("fishbowl",	DT_PROTECT,  TK_INTEGER,  0)
    D("flavor",		DT_CONSTANT, TK_STRING,	  "")
    F("for",		DT_CONSTANT, TK_FUNCTION, func_for)
    F("forall",		DT_CONSTANT, TK_FUNCTION, func_forall)
    F("forget_item",	DT_CONSTANT, TK_FUNCTION, func_forget_item)
    D("forgotten",	0,           TK_INTEGER,  0)
    E("freeze_item",	DT_CONSTANT, LIB_CNFADM,  func_freeze_item)
    D("frozen",		0,           TK_INTEGER,  0)
    D("fw_edit",	DT_CONSTANT, TK_INTEGER,  0)
    D("fw_erase",	DT_CONSTANT, TK_INTEGER,  1)
    D("fw_hide",	DT_CONSTANT, TK_INTEGER,  1)
    D("fw_retitle",	DT_CONSTANT, TK_INTEGER,  1)
    D("fwlist",		DT_PROTECT,  TK_STRING,	  "")
    F("ge",		DT_CONSTANT, TK_FUNCTION, func_ge)
    F("get",		DT_CONSTANT, TK_FUNCTION, func_get)
    F("getcookie",	DT_CONSTANT, TK_FUNCTION, func_getcookie)
    D("gid",		DT_PROTECT,  TK_INTEGER,  -1)
    F("grep",		DT_CONSTANT, TK_FUNCTION, func_grep)
    F("groupid",	DT_CONSTANT, TK_FUNCTION, func_groupid)
    F("groupname",	DT_CONSTANT, TK_FUNCTION, func_groupname)
    F("groups",		DT_CONSTANT, TK_FUNCTION, func_groups)
    F("gt",		DT_CONSTANT, TK_FUNCTION, func_gt)
    F("halt",		DT_CONSTANT, TK_FUNCTION, func_halt)
    D("hidden",		0,	     TK_INTEGER,  0)
    E("hide_resp",	DT_CONSTANT, LIB_CNFADM,  func_hide_resp)
    D("homedir",	DT_PROTECT,  TK_STRING,   "")
    D("http_content_type",0,	     TK_STRING,   "text/html")
    D("http_expires",	0,	     TK_TIME,	  0)
    D("http_headers",	0,	     TK_INTEGER,  1)
    D("http_location",	0,	     TK_STRING,   "")
    D("http_no_cache",	0,	     TK_INTEGER,   0)
    D("id",		DT_PROTECT,  TK_STRING,   "")
    F("if",		DT_CONSTANT, TK_FUNCTION, func_if)
    F("ifelse",		DT_CONSTANT, TK_FUNCTION, func_ifelse)
    F("in",		DT_CONSTANT, TK_FUNCTION, func_in)
    F("in_sel",		DT_CONSTANT, TK_FUNCTION, func_in_sel)
    F("inc",		DT_CONSTANT, TK_FUNCTION, func_inc)
    F("include",	DT_CONSTANT, TK_FUNCTION, func_include)
    F("index",		DT_CONSTANT, TK_FUNCTION, func_index)
    F("ingroup",	DT_CONSTANT, TK_FUNCTION, func_ingroup)
    F("inlist",		DT_CONSTANT, TK_FUNCTION, func_inlist)
    D("isel",		DT_MULTI,    TK_STRING,   "")
    D("item",		0,	     TK_INTEGER,  0)
    F("jump",		DT_CONSTANT, TK_FUNCTION, func_jump)
    E("kill_item",	DT_CONSTANT, LIB_CNFADM,  func_kill_item)
    F("killsession",	DT_CONSTANT, TK_FUNCTION, func_killsession)
    F("last_in_conf",	DT_CONSTANT, TK_FUNCTION, func_last_in_conf)
    D("lastdate",	0,	     TK_TIME,     0)
    F("le",		DT_CONSTANT, TK_FUNCTION, func_le)
    F("length",		DT_CONSTANT, TK_FUNCTION, func_length)
    F("line",		DT_CONSTANT, TK_FUNCTION, func_line)
    E("link_item",	DT_CONSTANT, LIB_CNFADM,  func_link_item)
    D("linkdate",	0,	     TK_TIME,     0)
    D("linked",		0,	     TK_INTEGER,  0)
    F("loadvar",	DT_CONSTANT, TK_FUNCTION, func_loadvar)
    D("loginlen",	DT_CONSTANT, TK_INTEGER,  MAX_LOGIN_LEN)
    D("loglevel",	DT_CONSTANT, TK_INTEGER,  0)
    F("loop",		DT_CONSTANT, TK_FUNCTION, func_loop)
    F("lt",		DT_CONSTANT, TK_FUNCTION, func_lt)
    D("mark",		DT_CONSTANT, TK_MARK,     0)
    F("mark_unseen",	DT_CONSTANT, TK_FUNCTION, func_mark_unseen)
    D("maxitem",	0,	     TK_INTEGER,  0)
    D("maxread",	0,	     TK_INTEGER,  0)
    D("maxresp",	0,           TK_INTEGER,  0)
    D("mayedit",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayerase",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayfreeze",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayhide",	DT_PROTECT,  TK_INTEGER,  0)
    D("maykill",	DT_PROTECT,  TK_INTEGER,  0)
    D("maypost",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayread",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayretire",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayretitle",	DT_PROTECT,  TK_INTEGER,  0)
    D("mayseealias",	DT_PROTECT,  TK_INTEGER,  1)
    D("mayseefname",	DT_PROTECT,  TK_INTEGER,  1)
    F("mod",		DT_CONSTANT, TK_FUNCTION, func_mod)
    F("ne",		DT_CONSTANT, TK_FUNCTION, func_ne)
    F("neg",		DT_CONSTANT, TK_FUNCTION, func_neg)
    E("new_conf",	DT_CONSTANT, LIB_CNFADM,  func_new_conf)
    F("newgroup",	DT_CONSTANT, TK_FUNCTION, func_newgroup)
    F("newgroupmem",	DT_CONSTANT, TK_FUNCTION, func_newgroupmem)
    F("newsession",	DT_CONSTANT, TK_FUNCTION, func_newsession)
    F("newuser",	DT_CONSTANT, TK_FUNCTION, func_newuser)
    D("newuseropen",	DT_CONSTANT, TK_INTEGER,  2)
    F("next_conf",	DT_CONSTANT, TK_FUNCTION, func_next_conf)
    F("next_int",	DT_CONSTANT, TK_FUNCTION, func_next_int)
    F("next_item",	DT_CONSTANT, TK_FUNCTION, func_next_item)
    F("next_partic",	DT_CONSTANT, TK_FUNCTION, func_next_partic)
    F("next_resp",	DT_CONSTANT, TK_FUNCTION, func_next_resp)
    F("nextdir",	DT_CONSTANT, TK_FUNCTION, func_nextdir)
    F("nextuser",	DT_CONSTANT, TK_FUNCTION, func_nextuser)
    D("nopwedit",	DT_CONSTANT, TK_INTEGER,  NOPWEDIT)
    D("noskip",		0,           TK_INTEGER,  0)
    F("ogrep",		DT_CONSTANT, TK_FUNCTION, func_ogrep)
    F("open_conf",	DT_CONSTANT, TK_FUNCTION, func_open_conf)
    F("or",		DT_CONSTANT, TK_FUNCTION, func_or)
    D("parentresp",	0,	     TK_INTEGER,  0)
    F("parse",		DT_CONSTANT, TK_FUNCTION, func_parse)
    D("particip",	DT_PROTECT,  TK_STRING,   "")
    F("pathname",	DT_CONSTANT, TK_FUNCTION, func_pathname)
    F("pop",		DT_CONSTANT, TK_FUNCTION, func_pop)
    F("post_item",	DT_CONSTANT, TK_FUNCTION, func_post_item)
    F("post_resp",	DT_CONSTANT, TK_FUNCTION, func_post_resp)
    F("pr",		DT_CONSTANT, TK_FUNCTION, func_pr)
    F("print",		DT_CONSTANT, TK_FUNCTION, func_print)
    D("progname",	DT_PROTECT,  TK_STRING,   "")
    F("put",		DT_CONSTANT, TK_FUNCTION, func_put)
    F("quote",		DT_CONSTANT, TK_FUNCTION, func_quote)
    F("rand",		DT_CONSTANT, TK_FUNCTION, func_rand)
    F("read",		DT_CONSTANT, TK_FUNCTION, func_read)
    F("read_item",	DT_CONSTANT, TK_FUNCTION, func_read_item)
    F("read_resp",	DT_CONSTANT, TK_FUNCTION, func_read_resp)
    F("read_text",	DT_CONSTANT, TK_FUNCTION, func_read_text)
    F("readable",	DT_CONSTANT, TK_FUNCTION, func_readable)
    D("readdate",	0,           TK_TIME,     0)
    F("regex",		DT_CONSTANT, TK_FUNCTION, func_regex)
    F("remember_item",	DT_CONSTANT, TK_FUNCTION, func_remember_item)
    F("removeuser",	DT_CONSTANT, TK_FUNCTION, func_removeuser)
    F("repeat",		DT_CONSTANT, TK_FUNCTION, func_repeat)
    F("replace",	DT_CONSTANT, TK_FUNCTION, func_replace)
    F("resign_conf",	DT_CONSTANT, TK_FUNCTION, func_resign_conf)
    D("resp",		0,	     TK_INTEGER,  0)
    E("retire_item",	DT_CONSTANT, LIB_CNFADM,  func_retire_item)
    D("retired",	0,           TK_INTEGER,  0)
    E("retitle_item",	DT_CONSTANT, LIB_CNFADM,  func_retitle_item)
    F("rev_sel",	DT_CONSTANT, TK_FUNCTION, func_rev_sel)
    F("rgrep",		DT_CONSTANT, TK_FUNCTION, func_rgrep)
    F("roll",		DT_CONSTANT, TK_FUNCTION, func_roll)
    D("rsel",		DT_MULTI,    TK_STRING,   "")
    F("savevar",	DT_CONSTANT, TK_FUNCTION, func_savevar)
    D("scriptdir",	DT_CONSTANT, TK_STRING,   SCRIPT_DIR)
    F("search",		DT_CONSTANT, TK_FUNCTION, func_search)
    D("secret",		DT_PROTECT,  TK_INTEGER,  0)
    F("seekuser",	DT_CONSTANT, TK_FUNCTION, func_seekuser)
    F("select",		DT_CONSTANT, TK_FUNCTION, func_select)
    F("selectuser",	DT_CONSTANT, TK_FUNCTION, func_selectuser)
    D("sessions",	DT_CONSTANT, TK_INTEGER,  SESSIONS)
    F("setcookie",	DT_CONSTANT, TK_FUNCTION, func_setcookie)
    F("sgrep",		DT_CONSTANT, TK_FUNCTION, func_sgrep)
    D("showforgotten",	0,	     TK_INTEGER,  0)
    F("showopt",	DT_CONSTANT, TK_FUNCTION, func_showopt)
    D("since",		0,	     TK_TIME,     0)
    F("sleep",		DT_CONSTANT, TK_FUNCTION, func_sleep)
    D("spell_lang",	0,	     TK_STRING,   "")
    F("spellcheck",	DT_CONSTANT, TK_FUNCTION, func_spellcheck)
    F("split",		DT_CONSTANT, TK_FUNCTION, func_split)
    F("srand",		DT_CONSTANT, TK_FUNCTION, func_srand)
    D("stack_limit",	0,	     TK_INTEGER,  0)
    F("stop",		DT_CONSTANT, TK_FUNCTION, func_stop)
    F("stopped",	DT_CONSTANT, TK_FUNCTION, func_stopped)
    F("store",		DT_CONSTANT, TK_FUNCTION, func_store)
    F("substr",		DT_CONSTANT, TK_FUNCTION, func_substr)
    D("text",		0,	     TK_STRING,   "")
    D("texttype",	0,	     TK_STRING,   "")
    F("this_item",	DT_CONSTANT, TK_FUNCTION, func_this_item)
    F("time",		DT_CONSTANT, TK_FUNCTION, func_time)
    F("timeout",	DT_CONSTANT, TK_FUNCTION, func_timeout)
    D("timezone",	0,	     TK_STRING,   "")
    D("title",		0,	     TK_STRING,   "")
    D("uid",		DT_PROTECT,  TK_INTEGER,  0)
    F("undef",		DT_CONSTANT, TK_FUNCTION, func_undef)
    F("unhtml",		DT_CONSTANT, TK_FUNCTION, func_unhtml)
    D("urlarg",		0,           TK_STRING,   "")
    F("userinfo",	DT_CONSTANT, TK_FUNCTION, func_userinfo)
    D("userlist",	DT_PROTECT,  TK_INTEGER,  0)
    F("validate",	DT_CONSTANT, TK_FUNCTION, func_validate)
    F("version",	DT_CONSTANT, TK_FUNCTION, func_version)
    F("while",		DT_CONSTANT, TK_FUNCTION, func_while)
    F("wrap",		DT_CONSTANT, TK_FUNCTION, func_wrap)
    F("writable",	DT_CONSTANT, TK_FUNCTION, func_writable)
    F("write",		DT_CONSTANT, TK_FUNCTION, func_write)
    F("xdef",		DT_CONSTANT, TK_FUNCTION, func_xdef)
    F("xdefconstant",	DT_CONSTANT, TK_FUNCTION, func_xdefconstant)
    F("xstore",		DT_CONSTANT, TK_FUNCTION, func_xstore)
    F("ztime",		DT_CONSTANT, TK_FUNCTION, func_ztime)
    F("~",		DT_CONSTANT, TK_FUNCTION, func_bnot)
    /* End of array marker */
    D("",		0,	     0,		  NULL) };
